#!/usr/bin/perl
#
# unregisterClient.pl
#
# Copyright (C) 2011-2014 Cesnet z.s.p.o
#
# Use of this source is governed by a BSD-style license, see LICENSE file.

use strict;
use warnings;

use Getopt::Std;
use Switch;
use DBI;
use DBD::mysql;
use DateTime;
use Sys::Syslog qw(:DEFAULT setlogsock);
Sys::Syslog::setlogsock('unix');
use Carp;
use FindBin qw($RealBin $RealScript);
FindBin::again();
use lib "$RealBin/../lib";
use WardenCommon;



################################################################################
#				VARIABLES
################################################################################
our $VERSION = "2.2";
my $etc = "$RealBin/../etc";

# read config file
my $conf_file = "$etc/warden-server.conf";
WardenCommon::loadConf($conf_file);



################################################################################
#                               FUNCTIONS
################################################################################
sub usage {
  print "Usage: $RealScript -i <client_id> [-h -V]\n";
  exit 1;
}


sub help {
  print "$RealScript -i <client_id> [-h -V]\n";
  print "-i     client_id for unregistration\n";
  print "-h     print this text and exit\n";
  print "-V     print version and exit\n";
  exit 0;
}


sub version {
  print "Current version: $VERSION\n";
  exit 0;
}


#-------------------------------------------------------------------------------
# sendMsg - wrapper for more complex WardenCommon::sendMsg function
#-------------------------------------------------------------------------------
sub sendMsg
{
  my $severity          = shift;
  my $syslog_msg        = shift;

  WardenCommon::sendMsg($WardenCommon::SYSLOG, $WardenCommon::SYSLOG_VERBOSE, $WardenCommon::SYSLOG_FACILITY, $severity,
                        $syslog_msg, undef, $RealScript);
}



################################################################################
#                               MAIN
################################################################################
our ($opt_i, $opt_h, $opt_V);

if ($#ARGV == -1) {
  usage;
}

die usage unless getopts("i:hV");
my $client_id		= $opt_i;
my $help        	= $opt_h;
my $script_version	= $opt_V;

# catch help option
if ($help) {
  help;
}

# catch version option
if ($script_version) {
	  version;
}

# check parameters definition
if (!defined $client_id) {
  WardenCommon::errMsg("Parameter 'client_id' is not defined!");
}

# connect to DB
my $dbh = WardenCommon::connectDB($WardenCommon::DB_NAME, $WardenCommon::DB_HOST, $WardenCommon::DB_USER, $WardenCommon::DB_PASS);

my $warden_server = WardenCommon::trim(`hostname -f`);

# check if client has been already registered
my $sth = $dbh->prepare("SELECT client_id, hostname, service, client_type, type FROM clients WHERE client_id = ? AND valid = 't' LIMIT 1;") or die "Cannot prepare statement: " . $dbh->errstr;
$sth->execute($client_id) or die "Cannot execute statement: " . $sth->errstr;
my ($id, $hostname, $service, $client_type, $type) = $sth->fetchrow();

# delete registered client
if (!defined $id) {
  sendMsg("err", "Attempt to delete unregister client [Client_id: '$client_id'] from Warden server '$warden_server'");
  WardenCommon::errMsg("Client (#$client_id) is not registered at Warden server '$warden_server'");
} else {
  if ($client_type eq 's') {
    $sth = $dbh->prepare("UPDATE clients SET valid = 'f' WHERE client_id = ?;") or die "Cannot prepare statement: " . $dbh->errstr;
    my $ret_val = $sth->execute($client_id) or die "Cannot execute statement: " . $sth->errstr;
    if ($ret_val == 1) {
      $sth = $dbh->prepare("UPDATE events SET valid = 'o' WHERE client_id = ?;") or die "Cannot prepare statement: " . $dbh->errstr;
      $sth->execute($client_id) or die "Cannot execute statement: " . $sth->errstr;
      # log message until after invalidate all events
      sendMsg("info", "Sender [Client_id: '$client_id', Hostname: '$hostname', Service: '$service'] has been deleted and its data were invalidated from Warden server '$warden_server'");
      WardenCommon::succMsg("Unregistration of sender (#$client_id) was SUCCESSFUL!!!");
    } else {
      WardenCommon::errMsg("Unregistration of sender (#$client_id) FAILED!!!");
    }
  } else {
    $sth = $dbh->prepare("UPDATE clients SET valid = 'f' WHERE client_id = ?;") or die "Cannot prepare statement: " . $dbh->errstr;
    my $ret_val = $sth->execute($client_id) or die "Cannot execute statement: " . $sth->errstr;
    if ($ret_val == 1) {
      sendMsg("info", "Receiver [Client_id: '$client_id', Hostname: '$hostname', Type: '$type'] has been deleted from Warden server '$warden_server'");
      WardenCommon::succMsg("Unregistration of receiver (#$client_id) was SUCCESSFUL!!!");
    } else {
      WardenCommon::errMsg("Unregistration of receiver (#$client_id) FAILED!!!");
    }
  }
}

# cleanup section
END {
  if (defined $dbh) {
    $dbh->disconnect or warn "Disconnection from database '$WardenCommon::DB_NAME' failed: $dbh->errstr\n";
  }
}

exit 0;
