#!/usr/bin/perl
#
# registerReceiver.pl
#
# Copyright (C) 2011-2014 Cesnet z.s.p.o
#
# Use of this source is governed by a BSD-style license, see LICENSE file.

use strict;
use warnings;

use Getopt::Std;
use Switch;
use DBI;
use DBD::mysql;
use DateTime;
use Sys::Syslog qw(:DEFAULT setlogsock);
Sys::Syslog::setlogsock('unix');
use Carp;
use FindBin qw($RealBin $RealScript);
FindBin::again();
use lib "$RealBin/../lib";
use WardenCommon;



################################################################################
#				VARIABLES
################################################################################
our $VERSION = "2.2";
my $etc = "$RealBin/../etc";

# read config file
my $conf_file = "$etc/warden-server.conf";
WardenCommon::loadConf($conf_file);



################################################################################
#                               FUNCTIONS
################################################################################
sub usage {
  print "Usage: $RealScript -n <hostname> -r <requestor> -t <type> -o -c <ip_net_client> [-i <client_id> -h -V]\n";
  exit 1;
}


sub help {
  print "$RealScript -n <hostname> -r <requestor> -t <type> -o -c <ip_net_client> [-i <client_id> -h -V]\n";
  print "-n     hostname of receiver\n";
  print "-r     e-mail address of client registration requestor\n";
  print "-t     type of received events or '_any_' for receiving of all types of events\n";
  print "-o     enable receiving of own events\n";
  print "-c     CIDR of receiver\n";
  print "-i     user defined client ID (optional)\n";
  print "-h     print this text and exit\n";
  print "-V     print version and exit\n";
  exit 0;
}


sub version {
  print "Current version: $VERSION\n";
  exit 0;
}



#-------------------------------------------------------------------------------
# sendMsg - wrapper for more complex WardenCommon::sendMsg function
#-------------------------------------------------------------------------------
sub sendMsg
{
  my $severity          = shift;
  my $syslog_msg        = shift;

  WardenCommon::sendMsg($WardenCommon::SYSLOG, $WardenCommon::SYSLOG_VERBOSE, $WardenCommon::SYSLOG_FACILITY, $severity,
                        $syslog_msg, undef, $RealScript);
}



################################################################################
#                               MAIN
################################################################################
our ($opt_n, $opt_r, $opt_t, $opt_o, $opt_c, $opt_i, $opt_h, $opt_V);

if ($#ARGV == -1) {
  usage;
}

die usage unless getopts("n:r:t:c:i:ohV");
my $hostname            = $opt_n;
my $requestor           = $opt_r;
my $type		= $opt_t;
my $ip_net_client       = $opt_c;
my $client_id		= $opt_i;
my $help                = $opt_h;
my $script_version	= $opt_V;
my $receive_own_events	= "f";
if ($opt_o) {
  $receive_own_events = "t";
}

# catch help option
if ($help) {
  help;
}

# catch version option
if ($script_version) {
  version;
}

# check parameters definition
switch () {
  case {!defined $hostname} 		{WardenCommon::errMsg("Parameter 'hostname' is not defined!")}
  case {!defined $requestor} 		{WardenCommon::errMsg("Parameter 'requestor' is not defined!")}
  case {!defined $type}			{WardenCommon::errMsg("Parameter 'type' is not defined!")}
  case {!defined $receive_own_events}	{WardenCommon::errMsg("Parameter 'receive_own_events' is not defined!")}
  case {!defined $ip_net_client} 	{WardenCommon::errMsg("Parameter 'ip_net_client' is not defined!")}
}

# check validity of IP address (CIDR format)
if ($ip_net_client !~ /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/(\d|[1-2]\d|3[0-2]))$/) {
  WardenCommon::errMsg("Enter correct IP address in CIDR format!");
}

# check client ID
if (defined $client_id && $client_id !~ /^\d+\z/) {
  WardenCommon::errMsg("Parameter 'client_id' is not a integer!");
}

# defined variables by script
my $client_type		= "r";
my $valid		= "t";
my $registered		= DateTime->now;
my $service		= undef;
my $description_tags	= undef;
my $warden_server = WardenCommon::trim(`hostname -f`);

# connect to DB
my $dbh = WardenCommon::connectDB($WardenCommon::DB_NAME, $WardenCommon::DB_HOST, $WardenCommon::DB_USER, $WardenCommon::DB_PASS);

# register receiver
my ($sth, $client_id_db, $hostname_db, $registered_db, $requestor_db, $client_type_db, $type_db, $receive_own_events_db, $ip_net_client_db, $valid_db);
# check if user client_id is specified and prepare statement according to this
if (defined $client_id) {
  $sth = $dbh->prepare("SELECT client_id, hostname, registered, requestor, client_type, type, receive_own_events, ip_net_client, valid FROM clients WHERE client_id = ?;") or die "Cannot prepare statement: " . $dbh->errstr;
  $sth->execute($client_id) or die "Cannot execute statement: " . $sth->errstr;
} else {
  $sth = $dbh->prepare("SELECT client_id, hostname, registered, requestor, client_type, type, receive_own_events, ip_net_client, valid FROM clients WHERE hostname = ? AND client_type = ? AND type = ? AND ip_net_client = ? LIMIT 1;") or die "Cannot prepare statement: " . $dbh->errstr;
  $sth->execute($hostname, $client_type, $type, $ip_net_client) or die "Cannot execute statement: " . $sth->errstr;
}
($client_id_db, $hostname_db, $registered_db, $requestor_db, $client_type_db, $type_db, $receive_own_events_db, $ip_net_client_db, $valid_db) = $sth->fetchrow();
# register receiver without user specified client_id
if (!defined $client_id_db) {
  $sth = $dbh->prepare("INSERT INTO clients VALUES (?,?,?,?,?,?,?,?,?,?,?);") or die "Cannot prepare statement: " . $dbh->errstr;
  my $ret_val = $sth->execute($client_id, $hostname, $registered, $requestor, $service, $client_type, $type, $receive_own_events, $description_tags, $ip_net_client, $valid) or die "Cannot execute statement: " . $sth->errstr;
  if ($ret_val == 1) {
    if (!defined $client_id) {
      $client_id = $dbh->last_insert_id(undef, undef, "clients", undef);
    }
    sendMsg("info", "New receiver [ID: '$client_id', Hostname: '$hostname', Requestor: '$requestor', Type: '$type', ROE: '$receive_own_events', CIDR: '$ip_net_client'] was registered");
    WardenCommon::succMsg("Registration of receiver (#$client_id) was SUCCESSFUL!");
  } else {
    WardenCommon::errMsg("Registration of receiver $hostname ($service) FAILED!");
  }
# re-activate old sender if is already registered but is not valid
} elsif (defined $client_id_db && $hostname eq $hostname_db && $client_type eq $client_type_db && $type eq $type_db && $ip_net_client eq $ip_net_client_db  && $valid_db eq 'f') {
  # change requestor if is needed
  if ($requestor ne $requestor_db) {
    $sth = $dbh->prepare("UPDATE clients SET requestor = ? WHERE client_id = ?;") or die "Cannot prepare statement: " . $dbh->errstr;
    $sth->execute($requestor, $client_id_db) or die "Cannot execute statement: " . $sth->errstr;
  }
  # change receive_own_events if is needed
  if ($receive_own_events ne $receive_own_events_db) {
    $sth = $dbh->prepare("UPDATE clients SET receive_own_events = ? WHERE client_id = ?;") or die "Cannot prepare statement: " . $dbh->errstr;
    $sth->execute($receive_own_events, $client_id_db) or die "Cannot execute statement: " . $sth->errstr;
  }
  # reactivate old receiver
  $sth = $dbh->prepare("UPDATE clients SET valid = 't' WHERE client_id = ?;") or die "Cannot prepare statement: " . $dbh->errstr;
  my $ret_val = $sth->execute($client_id_db) or die "Cannot execute statement: " . $sth->errstr;
  if ($ret_val == 1) {
    sendMsg("info", "Receiver [ID: '$client_id_db', Hostname: '$hostname', Requestor: '$requestor', Type: '$type', ROE: '$receive_own_events', CIDR: '$ip_net_client'] was re-activated");
    WardenCommon::succMsg("Re-activation of receiver (#$client_id_db) was SUCCESSFUL!");
  } else {
    WardenCommon::errMsg("Re-activation of receiver (#$client_id_db) FAILED!");
  }
# receiver is already registered and valid - send error msg
} elsif (defined $client_id_db && $hostname eq $hostname_db && $client_type eq $client_type_db && $type eq $type_db && $ip_net_client eq $ip_net_client_db  && $valid_db eq 't') {
  sendMsg("err", "Attempt to register receiver (#$client_id_db) registered at Warden server '$warden_server' in '$registered_db'");
  WardenCommon::errMsg("Receiver has been already registered at Warden server '$warden_server' in '$registered_db' with ID '$client_id_db'");
# user specified client_id is already used - send error msg
} else {
  sendMsg("err", "Attempt to use client_id (#$client_id_db) registered at Warden server '$warden_server' in '$registered_db'");
  WardenCommon::errMsg("Client ID (#$client_id_db) has been already used at Warden server '$warden_server' in '$registered_db'");
}

# cleanup section
END {
  if (defined $dbh) {
    $dbh->disconnect or warn "Disconnection from database '$WardenCommon::DB_NAME' failed: $dbh->errstr\n";
  }
}

exit 0;
