#!/usr/bin/perl
#
# registerSender.pl
#
# Copyright (C) 2011-2014 Cesnet z.s.p.o
#
# Use of this source is governed by a BSD-style license, see LICENSE file.

use strict;
use warnings;

use Getopt::Std;
use Switch;
use DBI;
use DBD::mysql;
use DateTime;
use Sys::Syslog qw(:DEFAULT setlogsock);
Sys::Syslog::setlogsock('unix');
use Carp;
use FindBin qw($RealBin $RealScript);
FindBin::again();
use lib "$RealBin/../lib";
use WardenCommon;



################################################################################
#				VARIABLES
################################################################################
our $VERSION = "2.2";
my $etc = "$RealBin/../etc";

# read config file
my $conf_file = "$etc/warden-server.conf";
WardenCommon::loadConf($conf_file);



################################################################################
#                               FUNCTIONS
################################################################################
sub usage {
  print "Usage: $RealScript -n <hostname> -r <requestor> -s <service> -d <description_tags> -c <cidr> [-i <client_id> -h -V]\n";
  exit 1;
}


sub help {
  print "$RealScript -n <hostname> -r <requestor> -s <service> -d <description_tags> -c <cidr> [-i <client_id> -h -V]\n";
  print "-n 	hostname of sender\n";
  print "-r 	e-mail address of client registration requestor\n";
  print "-s 	name of service which sent events\n";
  print "-d 	description tags of send events\n";
  print "-c 	CIDR of sender\n";
  print "-i	user defined client ID (optional)\n";
  print "-h 	print this text and exit\n";
  print "-V 	print version and exit\n";
  exit 0;
}


sub version {
  print "Current version: $VERSION\n";
  exit 0;
}


#-------------------------------------------------------------------------------
# sendMsg - wrapper for more complex WardenCommon::sendMsg function
#-------------------------------------------------------------------------------
sub sendMsg
{
  my $severity          = shift;
  my $syslog_msg        = shift;

  WardenCommon::sendMsg($WardenCommon::SYSLOG, $WardenCommon::SYSLOG_VERBOSE, $WardenCommon::SYSLOG_FACILITY, $severity,
                        $syslog_msg, undef, $RealScript);
}



################################################################################
#                               MAIN
################################################################################
our ($opt_n, $opt_r, $opt_s, $opt_d, $opt_c, $opt_i, $opt_h, $opt_V);

if ($#ARGV == -1) {
  usage;
}

die usage unless getopts("n:r:s:d:c:i:hV");
my $hostname		= $opt_n;
my $requestor		= $opt_r;
my $service		= $opt_s;
my $description_tags 	= $opt_d;
my $ip_net_client	= $opt_c;
my $client_id		= $opt_i;
my $help 		= $opt_h;
my $script_version	= $opt_V;

# catch help option
if ($help) {
  help;
}

# catch version option
if ($script_version) {
  version;
}

# check parameters definition
switch () {
  case {!defined $hostname} {WardenCommon::errMsg("Parameter '-n <hostname>' is not defined!")}
  case {!defined $requestor} {WardenCommon::errMsg("Parameter '-r <requestor>' is not defined!")}
  case {!defined $service} {WardenCommon::errMsg("Parameter '-s <service>' is not defined!")}
  case {!defined $description_tags} {WardenCommon::errMsg("Parameter '-d <description_tags>' is not defined!")}
  case {!defined $ip_net_client} {WardenCommon::errMsg("Parameter '-c <cidr>' is not defined!")}
}

# check validity of IP address (CIDR format)
if ($ip_net_client !~ /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])(\/(\d|[1-2]\d|3[0-2]))$/) {
  WardenCommon::errMsg("Enter correct IP in CIDR format!");
}

# check client ID
if (defined $client_id && $client_id !~ /^\d+\z/) {
  WardenCommon::errMsg("Parameter '-i <client_id>' is not a integer!");
}

# defined variables by method
my $client_type		= "s";
my $valid		= "t";
my $registered		= DateTime->now;
my $type		= undef;
my $receive_own_events	= undef;
my $warden_server = WardenCommon::trim(`hostname -f`);

# connect to DB
my $dbh = WardenCommon::connectDB($WardenCommon::DB_NAME, $WardenCommon::DB_HOST, $WardenCommon::DB_USER, $WardenCommon::DB_PASS);

# register sender
my ($sth, $client_id_db, $hostname_db, $registered_db, $requestor_db, $service_db, $client_type_db, $description_tags_db, $ip_net_client_db, $valid_db);
# check if user client_id is specified and prepare statement according to this
if (defined $client_id) {
  $sth = $dbh->prepare("SELECT client_id, hostname, registered, requestor, service, client_type, description_tags, ip_net_client, valid FROM clients WHERE client_id = ?;") or die "Cannot prepare statement: " . $dbh->errstr;
  $sth->execute($client_id) or die "Cannot execute statement: " . $sth->errstr;
} else {
  $sth = $dbh->prepare("SELECT client_id, hostname, registered, requestor, service, client_type, description_tags, ip_net_client, valid FROM clients WHERE hostname = ? AND service = ? AND client_type = ? AND ip_net_client = ? LIMIT 1;") or die "Cannot prepare statement: " . $dbh->errstr;
  $sth->execute($hostname, $service, $client_type, $ip_net_client) or die "Cannot execute statement: " . $sth->errstr;
}
($client_id_db, $hostname_db, $registered_db, $requestor_db, $service_db, $client_type_db, $description_tags_db, $ip_net_client_db, $valid_db) = $sth->fetchrow();
# register sender without user specified client_id
if (!defined $client_id_db) {
  $sth = $dbh->prepare("INSERT INTO clients VALUES (?,?,?,?,?,?,?,?,?,?,?);") or die "Cannot prepare statement: " . $dbh->errstr;
  my $ret_val = $sth->execute($client_id, $hostname, $registered, $requestor, $service, $client_type, $type, $receive_own_events, $description_tags, $ip_net_client, $valid) or die "Cannot execute statement: " . $sth->errstr;
  if ($ret_val == 1) {
    if (!defined $client_id) {
      $client_id = $dbh->last_insert_id(undef, undef, "clients", undef);
    }
    sendMsg("info", "New sender [ID: '$client_id', Hostname: '$hostname', Requestor: '$requestor', Service: '$service', Description_tags: '$description_tags', CIDR: '$ip_net_client'] was registered");
    WardenCommon::succMsg("Registration of sender (#$client_id) was SUCCESSFUL!");
  } else {
    WardenCommon::errMsg("Registration of sender $hostname ($service) FAILED!");
  }
# re-activate old sender if is already registered but is not valid
} elsif (defined $client_id_db && $hostname eq $hostname_db && $service eq $service_db && $client_type eq $client_type_db && $ip_net_client eq $ip_net_client_db  && $valid_db eq 'f') {
  # change requestor if is needed
  if ($requestor ne $requestor_db) {
    $sth = $dbh->prepare("UPDATE clients SET requestor = ? WHERE client_id = ?;") or die "Cannot prepare statement: " . $dbh->errstr;
    $sth->execute($requestor, $client_id_db) or die "Cannot execute statement: " . $sth->errstr;
  }
  # change description_tags if is needed
  if ($description_tags ne $description_tags_db) {
    $sth = $dbh->prepare("UPDATE clients SET description_tags = ? WHERE client_id = ?;") or die "Cannot prepare statement: " . $dbh->errstr;
    $sth->execute($description_tags, $client_id_db) or die "Cannot execute statement: " . $sth->errstr;
  }
  # re-activate old sender
  $sth = $dbh->prepare("UPDATE clients SET valid = 't' WHERE client_id = ?;") or die "Cannot prepare statement: " . $dbh->errstr;
  my $ret_val = $sth->execute($client_id_db) or die "Cannot execute statement: " . $sth->errstr;
  if ($ret_val == 1) {
    sendMsg("info", "Sender [ID: '$client_id_db', Hostname: '$hostname', Requestor: '$requestor', Service: '$service', Description_tags: '$description_tags', CIDR: '$ip_net_client'] was re-activated");
    WardenCommon::succMsg("Re-activation of sender (#$client_id_db) was SUCCESSFUL! (Client's old data are still obsoleted.)");
  } else {
    WardenCommon::errMsg("Re-activation of sender (#$client_id_db) FAILED!");
  }
# sender is already registered and valid - send error msg
} elsif (defined $client_id_db && $hostname eq $hostname_db && $service eq $service_db && $client_type eq $client_type_db && $ip_net_client eq $ip_net_client_db  && $valid_db eq 't') {
  sendMsg("err", "Attempt to register sender (#$client_id_db) registered at Warden server '$warden_server' in '$registered_db'");
  WardenCommon::errMsg("Sender has been already registered at Warden server '$warden_server' in '$registered_db' with ID '$client_id_db'");
# user specified client_id is already used - send error msg
} else {
  sendMsg("err", "Attempt to use client_id (#$client_id_db) registered at Warden server '$warden_server' in '$registered_db'");
  WardenCommon::errMsg("Client ID (#$client_id_db) has been already used at Warden server '$warden_server' in '$registered_db'");
}

# cleanup section
END {
  if (defined $dbh) {
    $dbh->disconnect or warn "Disconnection from database '$WardenCommon::DB_NAME' failed: $dbh->errstr\n";
  }
}

exit 0;
